@@ -163,74 +163,6 @@ ephy_gsb_service_schedule_update (EphyGSBService *self)
   LOG ("Next update scheduled in %ld seconds", interval);
 }
 
-static GList *
-ephy_gsb_service_fetch_threat_lists_sync (EphyGSBService *self)
-{
-  GList *retval = NULL;
-  JsonNode *body_node;
-  JsonObject *body_obj;
-  JsonArray *threat_lists;
-  JsonObject *descriptor;
-  const char *threat_type;
-  const char *platform_type;
-  const char *threat_entry_type;
-  SoupMessage *msg;
-  char *url;
-
-  g_assert (EPHY_IS_GSB_SERVICE (self));
-
-  url = g_strdup_printf ("%sthreatLists?key=%s", API_PREFIX, self->api_key);
-  msg = soup_message_new (SOUP_METHOD_GET, url);
-  soup_session_send_message (self->session, msg);
-
-  if (msg->status_code != 200) {
-    LOG ("Failed to fetch the threat lists from the server, got: %u, %s",
-         msg->status_code, msg->response_body->data);
-    goto out;
-  }
-
-  body_node = json_from_string (msg->response_body->data, NULL);
-  if (!body_node || !JSON_NODE_HOLDS_OBJECT (body_node)) {
-    g_warning ("Response is not a valid JSON object");
-    goto out;
-  }
-
-  body_obj = json_node_get_object (body_node);
-
-  if (json_object_has_non_null_array_member (body_obj, "threatLists")) {
-    threat_lists = json_object_get_array_member (body_obj, "threatLists");
-    for (guint i = 0; i < json_array_get_length (threat_lists); i++) {
-      descriptor = json_array_get_object_element (threat_lists, i);
-      threat_type = json_object_get_string_member (descriptor, "threatType");
-      platform_type = json_object_get_string_member (descriptor, "platformType");
-
-      /* Keep SOCIAL_ENGINEERING threats that are for any platform.
-       * Keep MALWARE/UNWANTED_SOFTWARE threats that are for Linux only.
-       */
-      if (g_strcmp0 (threat_type, "SOCIAL_ENGINEERING") == 0) {
-        if (g_strcmp0 (platform_type, "ANY_PLATFORM") != 0)
-          continue;
-      } else if (g_strcmp0 (platform_type, "LINUX") != 0) {
-          continue;
-      }
-
-      threat_entry_type = json_object_get_string_member (descriptor, "threatEntryType");
-      retval = g_list_prepend (retval, ephy_gsb_threat_list_new (threat_type,
-                                                                 platform_type,
-                                                                 threat_entry_type,
-                                                                 NULL));
-    }
-  }
-
-out:
-  g_free (url);
-  g_object_unref (msg);
-  if (body_node)
-    json_node_unref (body_node);
-
-  return g_list_reverse (retval);
-}
-
 static void
 ephy_gsb_service_update_thread (GTask          *task,
                                 EphyGSBService *self,
@@ -255,12 +187,6 @@ ephy_gsb_service_update_thread (GTask          *task,
 
   ephy_gsb_storage_delete_old_full_hashes (self->storage);
 
-  /* Fetch and store new threat lists, if any. */
-  threat_lists = ephy_gsb_service_fetch_threat_lists_sync (self);
-  for (GList *l = threat_lists; l && l->data; l = l->next)
-    ephy_gsb_storage_insert_threat_list (self->storage, l->data);
-  g_list_free_full (threat_lists, (GDestroyNotify)ephy_gsb_threat_list_free);
-
   threat_lists = ephy_gsb_storage_get_threat_lists (self->storage);
   if (!threat_lists) {
     LOG ("No threat lists to update");
@@ -352,8 +278,8 @@ ephy_gsb_service_update_thread (GTask          *task,
     double duration;
 
     duration_str = json_object_get_string_member (body_obj, "minimumWaitDuration");
-    /* Handle the trailing 's' character. */
-    sscanf (duration_str, "%lfs", &duration);
+    /* g_ascii_strtod() ignores trailing characters, i.e. 's' character. */
+    duration = g_ascii_strtod (duration_str, NULL);
     self->next_list_updates_time = CURRENT_TIME + (gint64)ceil (duration);
   }
 
@@ -373,7 +299,7 @@ ephy_gsb_service_update_finished_cb (EphyGSBService *self,
                                      GAsyncResult   *result,
                                      gpointer        user_data)
 {
-  self->is_updating = FALSE;
+  g_atomic_int_set (&self->is_updating, FALSE);
   g_signal_emit (self, signals[UPDATE_FINISHED], 0);
   ephy_gsb_service_schedule_update (self);
 }
@@ -386,7 +312,7 @@ ephy_gsb_service_update (EphyGSBService *self)
   g_assert (EPHY_IS_GSB_SERVICE (self));
   g_assert (ephy_gsb_storage_is_operable (self->storage));
 
-  self->is_updating = TRUE;
+  g_atomic_int_set (&self->is_updating, TRUE);
   task = g_task_new (self, NULL,
                      (GAsyncReadyCallback)ephy_gsb_service_update_finished_cb,
                      NULL);
@@ -640,7 +566,8 @@ ephy_gsb_service_update_full_hashes_sync (EphyGSBService *self,
       list = ephy_gsb_threat_list_new (threat_type, platform_type, threat_entry_type, NULL);
       hash = g_base64_decode (hash_b64, &length);
       positive_duration = json_object_get_string_member (match, "cacheDuration");
-      sscanf (positive_duration, "%lfs", &duration);
+      /* g_ascii_strtod() ignores trailing characters, i.e. 's' character. */
+      duration = g_ascii_strtod (positive_duration, NULL);
 
       ephy_gsb_storage_insert_full_hash (self->storage, list, hash, floor (duration));
 
@@ -651,14 +578,16 @@ ephy_gsb_service_update_full_hashes_sync (EphyGSBService *self,
 
   /* Update negative cache duration. */
   duration_str = json_object_get_string_member (body_obj, "negativeCacheDuration");
-  sscanf (duration_str, "%lfs", &duration);
+  /* g_ascii_strtod() ignores trailing characters, i.e. 's' character. */
+  duration = g_ascii_strtod (duration_str, NULL);
   for (GList *l = prefixes; l && l->data; l = l->next)
     ephy_gsb_storage_update_hash_prefix_expiration (self->storage, l->data, floor (duration));
 
   /* Handle minimum wait duration. */
   if (json_object_has_non_null_string_member (body_obj, "minimumWaitDuration")) {
     duration_str = json_object_get_string_member (body_obj, "minimumWaitDuration");
-    sscanf (duration_str, "%lfs", &duration);
+    /* g_ascii_strtod() ignores trailing characters, i.e. 's' character. */
+    duration = g_ascii_strtod (duration_str, NULL);
     self->next_full_hashes_time = CURRENT_TIME + (gint64)ceil (duration);
     ephy_gsb_storage_set_metadata (self->storage, "next_full_hashes_time", self->next_full_hashes_time);
   }
@@ -697,8 +626,13 @@ ephy_gsb_service_verify_url_thread (GTask          *task,
   /* If the local database is broken or an update is in course, we cannot
    * really verify the URL, so we have no choice other than to consider it safe.
    */
-  if (!ephy_gsb_storage_is_operable (self->storage) || self->is_updating) {
-    LOG ("Local GSB storage is not available at the moment, cannot verify URL");
+  if (g_atomic_int_get (&self->is_updating)) {
+    LOG ("Local GSB database is being updated, cannot verify URL");
+    goto out;
+  }
+
+  if (!ephy_gsb_storage_is_operable (self->storage)) {
+    LOG ("Local GSB database is broken, cannot verify URL");
     goto out;
   }
 
